# coding: utf8
# Copyright (c) Météo France (2017-)
# This software is governed by the CeCILL-C license under French law.
# http://www.cecill.info
""" This module gathers functions designed to store
scores computed from Evaluator objects in text files. """

from __future__ import print_function
import numpy as np
import pandas as pd
import evaltools as evt


def medianStationScores(objects, forecastDay, score_list, title=None,
                        outputFile=None, outputLatex=None, labels=None,
                        availability_ratio=0.75, min_nb_sta=10,
                        float_format=None):
    """ Built a table with median values of station scores.

    This function is based on Evaluator.stationScores method.
    Scores are first computed for each station at the wanted forecast day.
    Then, the median of these scores is taken.

    For daily time step data, scores are computed with one value per day of
    the period and per station.
    For hourly time step data, scores are computed with 24 values per day of
    the period and per station.

    Parameters
    ----------
    objects : list of evaltools.Evaluator objects
        Evaluator objects used for computation.
    forecastDay : int
        Forecast day corresponding to the data used in the calculation.
    score_list : list of str
        List of computed scores.
    title : str or None
        Title to add in the output file.
    outputFile : str
        File where to save the table. If None, the table is printed.
    outputLatex : str or None
        If not None, file where to save the table in a LaTeX layout.
    labels : None or list of str
        Labels of the objects.
    availability_ratio : float
        Minimal rate of data available on the period required per forecast
        time to compute the scores for each station.
    min_nb_sta : int
        Minimal number of stations required to compute the median of
        the scores.
    float_format : str
        String format for table values.

    """
    # build 'labels' from objects 'model' attribute if 'labels' is None
    if labels is None:
        labels = [obj.model for obj in objects]

    # compute stats
    stats = pd.DataFrame(columns=score_list)
    for obj, lab in zip(objects, labels):
        res = obj.stationScores(
            score_list=score_list,
            availability_ratio=availability_ratio)['D{}'.format(forecastDay)]
        idx_not_enough_values = np.sum(~res.isna(), axis=0) < min_nb_sta
        res = pd.DataFrame(np.nanmedian(res, axis=0), columns=[lab],
                           index=res.columns)
        res[idx_not_enough_values] = np.nan
        res = res.T
        stats = pd.concat([stats, res], sort=False)

    # print or save table
    if outputFile is None:
        print(stats.to_string(float_format=float_format))
    else:
        with open(outputFile, 'w') as f:
            if title is not None:
                f.write(title+'\n\n')
            f.write(stats.to_string(float_format=float_format))

    # save table with LaTeX layout
    if outputLatex is not None:
        with open(outputLatex, 'w') as f:
            if title is not None:
                f.write(title+'\n\n')
            f.write(stats.to_latex(bold_rows=True, float_format=float_format))


def exceedancesScores(objects, forecastDay, thresholds, score_list=None,
                      title=None, outputFile=None, outputLatex=None,
                      labels=None, float_format=None, start_end=None):
    """ Contingency table.

    Tables corresponding to the different thresholds are stored in the
    same file.
    Table values are computed with evaltools.scores.exceedancesScores
    function.

    Parameters
    ----------
    objects : list of evaltools.Evaluator objects
        Evaluator objects used for computation.
    forecastDay : int
            Forecast day corresponding to the data used in the calculation.
    thresholds : list of scalar
        Threshold values.
    score_list : list of str
        List of scores to display. If None, all available scores are displayed.
        Available scores are

          - 'accuracy' : Accuracy
          - 'bias_score' : Bias score
          - 'success_ratio' : Success ratio
          - 'hit_rate' : probability of detection (Hit rate)
          - 'false_alarm_ratio' : false alarm ratio
          - 'prob_false_detect' : probability of false detection
          - 'threat_score' : Threat Score
          - 'equitable_ts' : Equitable Threat Score
          - 'peirce_ss' : Peirce Skill Score (Hanssen and Kuipers discriminant)
          - 'heidke_ss' : Heidke Skill Score
          - 'rousseau_ss' : Rousseau Skill Score
          - 'odds_ratio' : Odds Ratio
          - 'odds_ratio_ss' : Odds Ratio Skill Score

    title : str or None
        Title to add in the output file.
    outputFile : str or None
        File where to save the tables. If None, the tables are printed.
    outputLatex : str or None
        If not None, file where to save the tables in a LaTeX layout.
    labels : None or list of str
        Labels of the objects.
    float_format : str
        String format for table values.
    start_end : None or list of two datetime.date objects
        Boundary dates used to select only data for a sub-period.

    """
    # build 'labels' from objects 'model' attribute if 'labels' is None
    if labels is None:
        labels = [obj.model for obj in objects]

    # check if we have daily values
    for obj in objects:
        if obj.seriesType not in ['daily']:
            print("*** Object attribute 'seriesType' must be 'daily' to " +
                  "use this method !!!")
            return

    # score_list default value
    if score_list is None:
        score_list = [
            'accuracy', 'bias_score', 'success_ratio', 'hit_rate',
            'false_alarm_ratio', 'prob_false_detect', 'threat_score',
            'equitable_ts', 'peirce_ss', 'heidke_ss', 'rousseau_ss',
            'odds_ratio', 'odds_ratio_ss',
        ]

    # compute stats
    tables = {}
    for thr in thresholds:
        res = pd.DataFrame()
        for obj, lab in zip(objects, labels):
            obs = obj.get_obs(forecastDay, start_end=start_end)
            obs = obs.values.flatten()
            sim = obj.get_sim(forecastDay, start_end=start_end)
            sim = sim.values.flatten()
            stats = evt.scores.exceedancesScores(obs, sim, thr=thr)[0]
            stats = {k: stats[k] if k in stats else np.nan for k in score_list}
            res = pd.concat([res, pd.DataFrame(stats, index=[lab])],
                            sort=False)
            tables[thr] = res

    # print or save tables
    if outputFile is None:
        if title is not None:
            print(title+'\n')
        for thr in thresholds:
            print("threshold = {}".format(thr))
            print(tables[thr].to_string(float_format=float_format))
            print('\n')
    else:
        with open(outputFile, 'w') as f:
            if title is not None:
                f.write(title+'\n\n')
            for thr in thresholds:
                f.write("threshold = {}\n".format(thr))
                f.write(tables[thr].to_string(float_format=float_format))
                f.write('\n\n')

    # save tables with LaTeX layout
    if outputLatex is not None:
        with open(outputLatex, 'w') as f:
            if title is not None:
                f.write(title+'\n\n')
            for thr in thresholds:
                f.write("threshold = {}\n".format(thr))
                f.write(tables[thr].to_latex(
                    bold_rows=True, float_format=float_format)
                )
                f.write('\n\n')

    return [tables[thr] for thr in thresholds]
